@@ -32,7 +32,7 @@ except ImportError:
 from openstackclient.common import exceptions
 from openstackclient.common import parseractions
 from openstackclient.common import utils
-from openstackclient.i18n import _  # noqa
+from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 
 
@@ -102,9 +102,10 @@ def _get_ip_address(addresses, address_type, ip_address_family):
                 if addy['OS-EXT-IPS:type'] == new_address_type:
                     if int(addy['version']) in ip_address_family:
                         return addy['addr']
+        msg = _("ERROR: No %(type)s IP version %(family)s address found")
         raise exceptions.CommandError(
-            "ERROR: No %s IP version %s address found" %
-            (address_type, ip_address_family)
+            msg % {"type": address_type,
+                   "family": ip_address_family}
         )
 
 
@@ -117,10 +118,7 @@ def _prep_server_detail(compute_client, server):
     """
     info = server._info.copy()
 
-    # Call .get() to retrieve all of the server information
-    # as findall(name=blah) and REST /details are not the same
-    # and do not return flavor and image information.
-    server = compute_client.servers.get(info['id'])
+    server = utils.find_resource(compute_client.servers, info['id'])
     info.update(server._info)
 
     # Convert the image blob to a name
@@ -155,6 +153,11 @@ def _prep_server_detail(compute_client, server):
     if 'tenant_id' in info:
         info['project_id'] = info.pop('tenant_id')
 
+    # Map power state num to meanful string
+    if 'OS-EXT-STS:power_state' in info:
+        info['OS-EXT-STS:power_state'] = _format_servers_list_power_state(
+            info['OS-EXT-STS:power_state'])
+
     # Remove values that are long and not too useful
     info.pop('links', None)
 
@@ -415,7 +418,11 @@ class CreateServer(command.ShowOne):
             try:
                 files[dst] = io.open(src, 'rb')
             except IOError as e:
-                raise exceptions.CommandError("Can't open '%s': %s" % (src, e))
+                msg = _("Can't open '%(source)s': %(exception)s")
+                raise exceptions.CommandError(
+                    msg % {"source": src,
+                           "exception": e}
+                )
 
         if parsed_args.min > parsed_args.max:
             msg = _("min instances should be <= max instances")
@@ -432,8 +439,11 @@ class CreateServer(command.ShowOne):
             try:
                 userdata = io.open(parsed_args.user_data)
             except IOError as e:
-                msg = "Can't open '%s': %s"
-                raise exceptions.CommandError(msg % (parsed_args.user_data, e))
+                msg = _("Can't open '%(data)s': %(exception)s")
+                raise exceptions.CommandError(
+                    msg % {"data": parsed_args.user_data,
+                           "exception": e}
+                )
 
         block_device_mapping = {}
         if volume:
@@ -552,7 +562,7 @@ class CreateServer(command.ShowOne):
             else:
                 self.log.error(_('Error creating server: %s'),
                                parsed_args.server_name)
-                sys.stdout.write(_('\nError creating server'))
+                sys.stdout.write(_('Error creating server\n'))
                 raise SystemExit
 
         details = _prep_server_detail(compute_client, server)
@@ -636,7 +646,7 @@ class CreateServerImage(command.ShowOne):
             else:
                 self.log.error(_('Error creating snapshot of server: %s'),
                                parsed_args.server)
-                sys.stdout.write(_('\nError creating server snapshot'))
+                sys.stdout.write(_('Error creating server snapshot\n'))
                 raise SystemExit
 
         image = _prep_image_detail(image_client, image_id)
@@ -678,7 +688,7 @@ class DeleteServer(command.Command):
                 else:
                     self.log.error(_('Error deleting server: %s'),
                                    server_obj.id)
-                    sys.stdout.write(_('\nError deleting server'))
+                    sys.stdout.write(_('Error deleting server\n'))
                     raise SystemExit
 
 
@@ -742,7 +752,8 @@ class ListServer(command.Lister):
         parser.add_argument(
             '--project',
             metavar='<project>',
-            help="Search by project (admin only) (name or ID)")
+            help=_("Search by project (admin only) (name or ID)")
+        )
         identity_common.add_project_domain_option_to_parser(parser)
         parser.add_argument(
             '--user',
@@ -760,19 +771,19 @@ class ListServer(command.Lister):
             '--marker',
             metavar='<marker>',
             default=None,
-            help=('The last server (name or ID) of the previous page. Display'
-                  ' list of servers after marker. Display all servers if not'
-                  ' specified.')
+            help=_('The last server (name or ID) of the previous page. Display'
+                   ' list of servers after marker. Display all servers if not'
+                   ' specified.')
         )
         parser.add_argument(
             '--limit',
             metavar='<limit>',
             type=int,
             default=None,
-            help=("Maximum number of servers to display. If limit equals -1,"
-                  " all servers will be displayed. If limit is greater than"
-                  " 'osapi_max_limit' option of Nova API,"
-                  " 'osapi_max_limit' will be used instead."),
+            help=_("Maximum number of servers to display. If limit equals -1,"
+                   " all servers will be displayed. If limit is greater than"
+                   " 'osapi_max_limit' option of Nova API,"
+                   " 'osapi_max_limit' will be used instead."),
         )
         return parser
 
@@ -1002,7 +1013,9 @@ class MigrateServer(command.Command):
             ):
                 sys.stdout.write(_('Complete\n'))
             else:
-                sys.stdout.write(_('\nError migrating server'))
+                self.log.error(_('Error migrating server: %s'),
+                               server.id)
+                sys.stdout.write(_('Error migrating server\n'))
                 raise SystemExit
 
 
@@ -1074,9 +1087,11 @@ class RebootServer(command.Command):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write(_('\nReboot complete\n'))
+                sys.stdout.write(_('Complete\n'))
             else:
-                sys.stdout.write(_('\nError rebooting server\n'))
+                self.log.error(_('Error rebooting server: %s'),
+                               server.id)
+                sys.stdout.write(_('Error rebooting server\n'))
                 raise SystemExit
 
 
@@ -1099,7 +1114,7 @@ class RebuildServer(command.ShowOne):
         parser.add_argument(
             '--password',
             metavar='<password>',
-            help="Set the password on the rebuilt instance",
+            help=_("Set the password on the rebuilt instance"),
         )
         parser.add_argument(
             '--wait',
@@ -1125,9 +1140,11 @@ class RebuildServer(command.ShowOne):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write(_('\nComplete\n'))
+                sys.stdout.write(_('Complete\n'))
             else:
-                sys.stdout.write(_('\nError rebuilding server'))
+                self.log.error(_('Error rebuilding server: %s'),
+                               server.id)
+                sys.stdout.write(_('Error rebuilding server\n'))
                 raise SystemExit
 
         details = _prep_server_detail(compute_client, server)
@@ -1279,7 +1296,9 @@ class ResizeServer(command.Command):
                 ):
                     sys.stdout.write(_('Complete\n'))
                 else:
-                    sys.stdout.write(_('\nError resizing server'))
+                    self.log.error(_('Error resizing server: %s'),
+                                   server.id)
+                    sys.stdout.write(_('Error resizing server\n'))
                     raise SystemExit
         elif parsed_args.confirm:
             compute_client.servers.confirm_resize(server)
@@ -1359,6 +1378,12 @@ class SetServer(command.Command):
             help=_('Property to add/change for this server '
                    '(repeat option to set multiple properties)'),
         )
+        parser.add_argument(
+            '--state',
+            metavar='<state>',
+            choices=['active', 'error'],
+            help=_('New server state (valid value: active, error)'),
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -1378,6 +1403,9 @@ class SetServer(command.Command):
                 parsed_args.property,
             )
 
+        if parsed_args.state:
+            server.reset_state(state=parsed_args.state)
+
         if parsed_args.root_password:
             p1 = getpass.getpass(_('New password: '))
             p2 = getpass.getpass(_('Retype new password: '))
@@ -1436,7 +1464,7 @@ class ShowServer(command.ShowOne):
         if parsed_args.diagnostics:
             (resp, data) = server.diagnostics()
             if not resp.status_code == 200:
-                sys.stderr.write(_("Error retrieving diagnostics data"))
+                sys.stderr.write(_("Error retrieving diagnostics data\n"))
                 return ({}, {})
         else:
             data = _prep_server_detail(compute_client, server)
