@@ -36,6 +36,9 @@ QUOTA = {
     "router": 10,
     "rbac_policy": -1,
     "port": 50,
+    "vip": 10,
+    "member": 10,
+    "health_monitor": 10,
 }
 
 
@@ -83,8 +86,7 @@ class FakeAddressScope(object):
         :return:
             A FakeResource object with name, id, etc.
         """
-        if attrs is None:
-            attrs = {}
+        attrs = attrs or {}
 
         # Set default attributes.
         address_scope_attrs = {
@@ -107,12 +109,49 @@ class FakeAddressScope(object):
 
         return address_scope
 
+    @staticmethod
+    def create_address_scopes(attrs=None, count=2):
+        """Create multiple fake address scopes.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :param int count:
+            The number of address scopes to fake
+        :return:
+            A list of FakeResource objects faking the address scopes
+        """
+        address_scopes = []
+        for i in range(0, count):
+            address_scopes.append(
+                FakeAddressScope.create_one_address_scope(attrs))
+
+        return address_scopes
+
+    @staticmethod
+    def get_address_scopes(address_scopes=None, count=2):
+        """Get an iterable MagicMock object with a list of faked address scopes.
+
+        If address scopes list is provided, then initialize the Mock object
+        with the list. Otherwise create one.
+
+        :param List address scopes:
+            A list of FakeResource objects faking address scopes
+        :param int count:
+            The number of address scopes to fake
+        :return:
+            An iterable Mock object with side_effect set to a list of faked
+            address scopes
+        """
+        if address_scopes is None:
+            address_scopes = FakeAddressScope.create_address_scopes(count)
+        return mock.MagicMock(side_effect=address_scopes)
+
 
 class FakeAvailabilityZone(object):
     """Fake one or more network availability zones (AZs)."""
 
     @staticmethod
-    def create_one_availability_zone(attrs={}):
+    def create_one_availability_zone(attrs=None):
         """Create a fake AZ.
 
         :param Dictionary attrs:
@@ -120,6 +159,8 @@ class FakeAvailabilityZone(object):
         :return:
             A FakeResource object with name, state, etc.
         """
+        attrs = attrs or {}
+
         # Set default attributes.
         availability_zone = {
             'name': uuid.uuid4().hex,
@@ -136,7 +177,7 @@ class FakeAvailabilityZone(object):
         return availability_zone
 
     @staticmethod
-    def create_availability_zones(attrs={}, count=2):
+    def create_availability_zones(attrs=None, count=2):
         """Create multiple fake AZs.
 
         :param Dictionary attrs:
@@ -155,19 +196,64 @@ class FakeAvailabilityZone(object):
         return availability_zones
 
 
+class FakeIPAvailability(object):
+    """Fake one or more network ip availabilities."""
+
+    @staticmethod
+    def create_one_ip_availability():
+        """Create a fake list with ip availability stats of a network.
+
+        :return:
+            A FakeResource object with network_name, network_id, etc.
+        """
+
+        # Set default attributes.
+        network_ip_availability = {
+            'network_id': 'network-id-' + uuid.uuid4().hex,
+            'network_name': 'network-name-' + uuid.uuid4().hex,
+            'tenant_id': '',
+            'subnet_ip_availability': [],
+            'total_ips': 254,
+            'used_ips': 6,
+        }
+
+        network_ip_availability = fakes.FakeResource(
+            info=copy.deepcopy(network_ip_availability),
+            loaded=True)
+        return network_ip_availability
+
+    @staticmethod
+    def create_ip_availability(count=2):
+        """Create fake list of ip availability stats of multiple networks.
+
+        :param int count:
+            The number of networks to fake
+        :return:
+            A list of FakeResource objects faking network ip availability stats
+        """
+        network_ip_availabilities = []
+        for i in range(0, count):
+            network_ip_availability = \
+                FakeIPAvailability.create_one_ip_availability()
+            network_ip_availabilities.append(network_ip_availability)
+
+        return network_ip_availabilities
+
+
 class FakeNetwork(object):
     """Fake one or more networks."""
 
     @staticmethod
-    def create_one_network(attrs={}):
+    def create_one_network(attrs=None):
         """Create a fake network.
 
         :param Dictionary attrs:
             A dictionary with all attributes
         :return:
-            A FakeResource object, with id, name, admin_state_up,
-            router_external, status, subnets, tenant_id
+            A FakeResource object, with id, name, etc.
         """
+        attrs = attrs or {}
+
         # Set default attributes.
         network_attrs = {
             'id': 'network-id-' + uuid.uuid4().hex,
@@ -178,7 +264,7 @@ class FakeNetwork(object):
             'shared': False,
             'subnets': ['a', 'b'],
             'provider_network_type': 'vlan',
-            'router_external': True,
+            'router:external': True,
             'availability_zones': [],
             'availability_zone_hints': [],
             'is_default': False,
@@ -192,11 +278,12 @@ class FakeNetwork(object):
 
         # Set attributes with special mapping in OpenStack SDK.
         network.project_id = network_attrs['tenant_id']
+        network.is_router_external = network_attrs['router:external']
 
         return network
 
     @staticmethod
-    def create_networks(attrs={}, count=2):
+    def create_networks(attrs=None, count=2):
         """Create multiple fake networks.
 
         :param Dictionary attrs:
@@ -232,20 +319,72 @@ class FakeNetwork(object):
         return mock.MagicMock(side_effect=networks)
 
 
+class FakeNetworkSegment(object):
+    """Fake one or more network segments."""
+
+    @staticmethod
+    def create_one_network_segment(attrs=None):
+        """Create a fake network segment.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object faking the network segment
+        """
+        attrs = attrs or {}
+
+        # Set default attributes.
+        network_segment_attrs = {
+            'id': 'segment-id-' + uuid.uuid4().hex,
+            'network_id': 'network-id-' + uuid.uuid4().hex,
+            'network_type': 'vlan',
+            'physical_network': 'physical-network-name-' + uuid.uuid4().hex,
+            'segmentation_id': 1024,
+        }
+
+        # Overwrite default attributes.
+        network_segment_attrs.update(attrs)
+
+        network_segment = fakes.FakeResource(
+            info=copy.deepcopy(network_segment_attrs),
+            loaded=True
+        )
+
+        return network_segment
+
+    @staticmethod
+    def create_network_segments(attrs=None, count=2):
+        """Create multiple fake network segments.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :param int count:
+            The number of network segments to fake
+        :return:
+            A list of FakeResource objects faking the network segments
+        """
+        network_segments = []
+        for i in range(0, count):
+            network_segments.append(
+                FakeNetworkSegment.create_one_network_segment(attrs)
+            )
+        return network_segments
+
+
 class FakePort(object):
     """Fake one or more ports."""
 
     @staticmethod
-    def create_one_port(attrs={}):
+    def create_one_port(attrs=None):
         """Create a fake port.
 
         :param Dictionary attrs:
             A dictionary with all attributes
-        :param Dictionary methods:
-            A dictionary with all methods
         :return:
             A FakeResource object, with id, name, etc.
         """
+        attrs = attrs or {}
+
         # Set default attributes.
         port_attrs = {
             'admin_state_up': True,
@@ -288,7 +427,7 @@ class FakePort(object):
         return port
 
     @staticmethod
-    def create_ports(attrs={}, count=2):
+    def create_ports(attrs=None, count=2):
         """Create multiple fake ports.
 
         :param Dictionary attrs:
@@ -328,7 +467,7 @@ class FakeRouter(object):
     """Fake one or more routers."""
 
     @staticmethod
-    def create_one_router(attrs={}):
+    def create_one_router(attrs=None):
         """Create a fake router.
 
         :param Dictionary attrs:
@@ -337,6 +476,8 @@ class FakeRouter(object):
             A FakeResource object, with id, name, admin_state_up,
             status, tenant_id
         """
+        attrs = attrs or {}
+
         # Set default attributes.
         router_attrs = {
             'id': 'router-id-' + uuid.uuid4().hex,
@@ -364,7 +505,7 @@ class FakeRouter(object):
         return router
 
     @staticmethod
-    def create_routers(attrs={}, count=2):
+    def create_routers(attrs=None, count=2):
         """Create multiple fake routers.
 
         :param Dictionary attrs:
@@ -412,8 +553,7 @@ class FakeSecurityGroup(object):
         :return:
             A FakeResource object, with id, name, etc.
         """
-        if attrs is None:
-            attrs = {}
+        attrs = attrs or {}
 
         # Set default attributes.
         security_group_attrs = {
@@ -467,16 +607,15 @@ class FakeSecurityGroupRule(object):
         :return:
             A FakeResource object, with id, etc.
         """
-        if attrs is None:
-            attrs = {}
+        attrs = attrs or {}
 
         # Set default attributes.
         security_group_rule_attrs = {
             'direction': 'ingress',
             'ethertype': 'IPv4',
             'id': 'security-group-rule-id-' + uuid.uuid4().hex,
-            'port_range_max': 0,
-            'port_range_min': 0,
+            'port_range_max': None,
+            'port_range_min': None,
             'protocol': 'tcp',
             'remote_group_id': None,
             'remote_ip_prefix': '0.0.0.0/0',
@@ -519,7 +658,7 @@ class FakeSubnet(object):
     """Fake one or more subnets."""
 
     @staticmethod
-    def create_one_subnet(attrs={}):
+    def create_one_subnet(attrs=None):
         """Create a fake subnet.
 
         :param Dictionary attrs:
@@ -527,6 +666,8 @@ class FakeSubnet(object):
         :return:
             A FakeResource object faking the subnet
         """
+        attrs = attrs or {}
+
         # Set default attributes.
         project_id = 'project-id-' + uuid.uuid4().hex
         subnet_attrs = {
@@ -551,13 +692,14 @@ class FakeSubnet(object):
 
         subnet = fakes.FakeResource(info=copy.deepcopy(subnet_attrs),
                                     loaded=True)
+
         # Set attributes with special mappings in OpenStack SDK.
         subnet.project_id = subnet_attrs['tenant_id']
 
         return subnet
 
     @staticmethod
-    def create_subnets(attrs={}, count=2):
+    def create_subnets(attrs=None, count=2):
         """Create multiple fake subnets.
 
         :param Dictionary attrs:
@@ -578,7 +720,7 @@ class FakeFloatingIP(object):
     """Fake one or more floating ip."""
 
     @staticmethod
-    def create_one_floating_ip(attrs={}):
+    def create_one_floating_ip(attrs=None):
         """Create a fake floating ip.
 
         :param Dictionary attrs:
@@ -586,6 +728,8 @@ class FakeFloatingIP(object):
         :return:
             A FakeResource object, with id, ip, and so on
         """
+        attrs = attrs or {}
+
         # Set default attributes.
         floating_ip_attrs = {
             'id': 'floating-ip-id-' + uuid.uuid4().hex,
@@ -614,7 +758,7 @@ class FakeFloatingIP(object):
         return floating_ip
 
     @staticmethod
-    def create_floating_ips(attrs={}, count=2):
+    def create_floating_ips(attrs=None, count=2):
         """Create multiple fake floating ips.
 
         :param Dictionary attrs:
@@ -653,7 +797,7 @@ class FakeSubnetPool(object):
     """Fake one or more subnet pools."""
 
     @staticmethod
-    def create_one_subnet_pool(attrs={}):
+    def create_one_subnet_pool(attrs=None):
         """Create a fake subnet pool.
 
         :param Dictionary attrs:
@@ -661,6 +805,8 @@ class FakeSubnetPool(object):
         :return:
             A FakeResource object faking the subnet pool
         """
+        attrs = attrs or {}
+
         # Set default attributes.
         subnet_pool_attrs = {
             'id': 'subnet-pool-id-' + uuid.uuid4().hex,
@@ -691,7 +837,7 @@ class FakeSubnetPool(object):
         return subnet_pool
 
     @staticmethod
-    def create_subnet_pools(attrs={}, count=2):
+    def create_subnet_pools(attrs=None, count=2):
         """Create multiple fake subnet pools.
 
         :param Dictionary attrs:
